<?php

namespace App\Jobs;

use App\Feedback;
use App\Mark;
use App\Wowa;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use RUCD\Training\SolutionDistance;

class ProcessAUC implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    protected $wowa;
    protected $data = [];
    protected $expected = [];
    protected $auc;
    protected $pr;

    /**
     * Create a new job instance.
     *
     * @param Wowa $wowa
     * @param bool $auc
     * @param bool $pr
     * @param array|null $data
     * @param array|null $expected
     */
    public function __construct(Wowa $wowa, bool $auc, bool $pr, array $data = null, array $expected = null)
    {
        $this->wowa = $wowa;
        $this->data = $data;
        $this->expected = $expected;
        $this->auc = $auc;
        $this->pr = $pr;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        $this->wowa->status = Wowa::STATE_RUNNING;
        $this->wowa->save();
        $solution = new SolutionDistance(count($this->wowa->w_weights));
        $solution->weights_w = $this->wowa->w_weights;
        $solution->weights_p = $this->wowa->p_weights;
        if ($this->data === null || $this->expected === null) {
            try {
                Feedback::chunk(100, function ($feedbacks) {
                    foreach ($feedbacks as $feed) {
                        $evidence_references = $feed->report()->references;
                        $this->expected[] = $feed->is_true_alert;
                        $scores_references = [];
                        foreach ($evidence_references as $evidence_reference) {
                            $scores_references[] = Mark::get()->findEvidenceById($evidence_reference)->score;
                        }
                        $this->data[] = $scores_references;
                    }
                });
            } catch (\Exception $e) {
                $this->wowa->status = Wowa::STATE_FAILED;
                $this->wowa->save();
            }
        }
        if ($this->auc) {
            $solution->computeAUC($this->data, $this->expected);
            $this->wowa->roc = $solution->getAucRoc();
        }
        if ($this->pr) {
            $solution->computePRAUC($this->data, $this->expected);
            $this->wowa->pr = $solution->getAucPr();
        }
        $this->wowa->status = Wowa::STATE_SUCCESS;
        $this->wowa->save();
    }
}
